'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function index (babel) {
  const template = babel.template;
  let exportDefaultDeclarationName = '';
  return {
    visitor: {
      Program: {
        enter(path) {
          if (!path.scope.hasBinding('createElement')) {
            path.unshiftContainer('body', template.ast(`import { createElement } from 'react';`));
          }

          if (!path.scope.hasBinding('ReactDOM')) {
            path.unshiftContainer('body', template.ast(`import ReactDOM from "react-dom";`));
          }
        },

        exit(path, state) {
          if (exportDefaultDeclarationName === '') return;
          const {
            opts: {
              root = 'app',
              hydrate = false
            }
          } = state;
          path.pushContainer('body', template.ast(`if (typeof getStaticProps !== "undefined") {
                (async function () {
                  const data = await getStaticProps();
                  ReactDOM.${hydrate ? 'hydrate' : 'render'}(createElement(${exportDefaultDeclarationName}, data.props), document.getElementById("${root}"));
                })();
              } else {
                ReactDOM.${hydrate ? 'hydrate' : 'render'}(createElement(${exportDefaultDeclarationName}), document.getElementById("${root}"));
              }`));
        }

      },
      ExportDefaultDeclaration: {
        enter(path) {
          // Only render function component
          if (path.node.declaration.type !== 'FunctionDeclaration') throw new Error('Only Function Component is supported'); // Anonymous function

          if (!path.node.declaration.id) throw new Error('Anonymous Function Component not supported'); // save for next

          exportDefaultDeclarationName = path.node.declaration.id.name;
        }

      }
    }
  };
}

exports.default = index;
